const initRiveText = require("../build/bin/debug/rive_text.js");

test("can init layout engine", async () => {
  const riveText = await initRiveText();
  const style = riveText.makeYogaStyle();
  expect(style).not.toBe(null);
  riveText.disposeYogaStyle(style);

  const node = riveText.makeYogaNode();
  expect(node).not.toBe(null);
  riveText.disposeYogaNode(node);
});

test("test layout values", async () => {
  const riveText = await initRiveText();
  const node = riveText.makeYogaNode();
  riveText.yogaNodeCalculateLayout(node, 1024, 768, 1);
  var layout = riveText.yogaNodeGetLayout(node);
  expect(layout.left).toBe(0);
  expect(layout.top).toBe(0);
  expect(layout.width).toBe(1024);
  expect(layout.height).toBe(768);
  riveText.disposeYogaNode(node);
});

test("test measure function", async () => {
  const riveText = await initRiveText();
  const root = riveText.makeYogaNode();
  const node = riveText.makeYogaNode();
  riveText.yogaNodeSetMeasureFunc(
    node,
    function (nodePtr, width, widthMode, height, heightMode) {
      return [120, 130];
    }
  );
  riveText.yogaNodeInsertChild(root, node, 0);

  riveText.yogaNodeCalculateLayout(root, 1024, 768, 1);
  var layout = riveText.yogaNodeGetLayout(node);
  expect(layout.left).toBe(0);
  expect(layout.top).toBe(0);
  expect(layout.width).toBe(1024); // because this is stretch
  expect(layout.height).toBe(130);

  riveText.yogaNodeClearMeasureFunc(node);
  riveText.yogaNodeMarkDirty(node);
  riveText.yogaNodeCalculateLayout(root, 1024, 768, 1);
  var layout = riveText.yogaNodeGetLayout(node);
  expect(layout.left).toBe(0);
  expect(layout.top).toBe(0);
  expect(layout.width).toBe(1024);
  expect(layout.height).toBe(0);

  riveText.disposeYogaNode(node);
  riveText.disposeYogaNode(root);
});
